﻿CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$
begin
	BEGIN
		execute query;
	EXCEPTION
		WHEN duplicate_table THEN
	             --do nothing we want to run all the way through;
        WHEN duplicate_object THEN
                 --do nothing we want to run all the way through;
        WHEN invalid_table_definition THEN
                 --do nothing we want to run all the way through;
        WHEN invalid_foreign_key THEN
                 --do nothing we want to run all the way through;

	END;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


create or replace function misc1()
returns VOID AS
$BODY$
begin

perform ex('

CREATE TABLE nominal_lookup
(
  nsuk serial NOT NULL,
  external_code character(20),
  nominal_code character(6),
  primary key (nsuk)
);

');

perform ex('create unique index ctnomlookup1 on nominal_lookup(external_code);');

perform ex('create unique index ctnomlookup2 on nominal_lookup(nominal_code);');

perform ex('alter table vat add primary key(cod);');

perform ex('
CREATE TABLE vat_lookup
(
  nsuk serial NOT NULL,
  vcode smallint,
  external_code integer,
  primary key (nsuk)
);

');

perform ex('create unique index ctvatlookup1 on vat_lookup(external_code);');
perform ex('create unique index ctvatlookup2 on vat_lookup(vcode);');

perform ex('alter table vat_lookup add constraint fk_vatlookup1 foreign key (vcode) references vat ');

perform ex('
CREATE TABLE non_disc_item
(
    nsuk serial NOT NULL,
    pdesc_asset_reg character(1),
    pdesc_id character(4),
    product_id integer,
    product_type_id integer,
    primary key (nsuk)
);
');

perform ex('alter table non_disc_item add constraint non_desc_fk1 foreign key (product_id) references product; ');
perform ex('alter table non_disc_item add constraint non_desc_fk2 foreign key (product_type_id) references product_type;');
perform ex('alter table non_disc_item add constraint non_desc_fk3 foreign key (pdesc_id) references pdesc (cod);');




end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc1() OWNER TO postgres;

select * from misc1();


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function misc20100510()
returns VOID AS
$BODY$

begin

--perform ex(' sql query here');

--perform ex(' sql query 2  in here ');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20100510() OWNER TO postgres;

-- Function: verify_dr_nl(date, integer)

-- DROP FUNCTION verify_dr_nl(date, integer);

CREATE OR REPLACE FUNCTION verify_dr_nl(theperiod date, psuppress_ok integer)
  RETURNS SETOF jp_verify AS
$BODY$
declare Pslsum          pledger.amount%type;
        Pnlsum          pledger.amount%type;
        Pdebtor         reserved_ac.account_code%type;
        Pcount          integer;
        Pnlperiod       nltrans.period%type;
        Pdirection      char(20);
        debtors_row     record;
        nominal_row     record;
        ret_row         jp_verify;
        w_cnt           integer;
        y_cnt           integer;
        count_row       record;
        this_period     sledger.period%type;

begin

    this_period := theperiod;

    select account_code
      into Pdebtor
      from reserved_ac
     where key_name = 'debtors';

    for debtors_row in
      select a.source, a.batch, sum(a.amount) as total
       from sledger a
      where a.period = this_period
        and a.cod is not null
      group by 1,2

    loop

        ret_row.direction := 'Sales to Nominal';

        select count(*)
          into Pcount
         from nltrans
        where source = debtors_row.source
          and batch = debtors_row.batch;

        if Pcount = 0 then
            ret_row.comment := '[REBUILD] Batch does not exist';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := null;
            ret_row.nominal_period := null;
            return next ret_row;
            continue;
        end if;

        select sum(amount)
         into Pnlsum
         from nltrans
        where source = debtors_row.source
          and batch = debtors_row.batch;

        if Pnlsum is null then
            Pnlsum := 0.00;
        end if;

        if Pnlsum <> 0.00 then
            ret_row.comment := '[REBUILD] Batch is not in balance';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := null;
            return next ret_row;
        end if;

        select sum(amount)
         into Pnlsum
         from nltrans
        where source = debtors_row.source
          and batch = debtors_row.batch
          and cod = Pdebtor;

         if Pnlsum is null then
            ret_row.comment := '[REBUILD] No Debtor Posting';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := null;
            return next ret_row;
        end if;


        if Pnlsum is null then
            Pnlsum := 0.00;
        end if;

        if Pnlsum <> debtors_row.total then
            ret_row.comment := 'Totals do not agree';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := null;
            return next ret_row;
        end if;

        w_cnt := 0;

        for count_row in
            select  distinct period
             from nltrans
             where source = debtors_row.source
              and batch = debtors_row.batch
        loop
            w_cnt := w_cnt + 1;
        end loop;

        y_cnt := 0;

        for count_row in
            select  distinct period
             from sledger
             where source = debtors_row.source
              and batch = debtors_row.batch
        loop
            y_cnt := y_cnt + 1;
        end loop;

        if w_cnt > 1 or y_cnt > 1 then
            if w_cnt > 1 then
                ret_row.comment := '[REBUILD] Batch posted to ' || w_cnt || ' periods';
            else
                ret_row.comment := '[SPLIT] Batch posted to ' || y_cnt || ' periods';
            end if;
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        else

            select distinct period
              into Pnlperiod
             from nltrans
            where source = debtors_row.source
              and batch = debtors_row.batch;

            if Pnlperiod <> ThePeriod then
                ret_row.comment := '[MOVE] Batch posted to a different period';
                ret_row.source := debtors_row.source;
                ret_row.batch := debtors_row.batch;
                ret_row.subledger_total := debtors_row.total;
                ret_row.nominal_total := Pnlsum;
                ret_row.nominal_period := Pnlperiod;
                return next ret_row;
            end if;
        end if;

        if Psuppress_ok <> 1 then
            ret_row.comment := 'Batch OK';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        end if;

    end loop;

    Pslsum := 0.00;

    for nominal_row in
     select source, batch, sum(amount) as total
       from nltrans
      where nltrans.period = this_period
        and nltrans.cod = Pdebtor
        and source <> 'CP'
      group by 1,2
      UNION
      select nltrans.source, batch, sum(amount) as total
       from nltrans , nlbatch
      where nltrans.source = nlbatch.source
        and nltrans.batch = nlbatch.seq
        and nltrans.period is null
        and nltrans.cod = Pdebtor
        and nltrans.source <> 'CP'
        and nlbatch.period = this_period
      group by 1,2

    loop

      ret_row.direction := 'Nominal To Sales';

	-- dont check sledger if the nominal debtor is zero
	if nominal_row.total <> 0 THEN

		select count(*)
		  into Pcount
		 from sledger
		where source = nominal_row.source
		  and batch = nominal_row.batch;

		Pslsum := 0.00;

		if Pcount = 0 THEN
		    ret_row.comment := '[NO FIX] Batch does not exist';
		    ret_row.source := nominal_row.source;
		    ret_row.batch := nominal_row.batch;
		    ret_row.subledger_total := 0.00;
		    ret_row.nominal_total := nominal_row.total;
		    ret_row.nominal_period := null;
		    return next ret_row;
		    continue;
		end if;



		select sum(amount)
		  into Pslsum
		  from sledger
		 where source = nominal_row.source
		   and batch = nominal_row.batch;

		 if nominal_row.total <> Pslsum then
		    ret_row.comment := '[REBUILD] Totals do not agree';
		    ret_row.source := nominal_row.source;
		    ret_row.batch := nominal_row.batch;
		    ret_row.subledger_total := 0.00;
		    ret_row.nominal_total := nominal_row.total;
		    ret_row.nominal_period := null;
		    return next ret_row;
		end if;

	end if;



        w_cnt := 0;

        for count_row in
            select distinct period
              from sledger
             where source = nominal_row.source
               and batch = nominal_row.batch
        loop
            w_cnt := w_cnt + 1;
        end loop;

        if w_cnt > 1 then
            ret_row.comment := '[SPLIT] Batch posted to ' || w_cnt || ' periods';
            ret_row.source := nominal_row.source;
            ret_row.batch := nominal_row.batch;
            ret_row.subledger_total := Pslsum;
            ret_row.nominal_total := nominal_row.total;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        else

            select distinct period
              into Pnlperiod                -- Use Pnlperiod to store the slperiod in this case
              from sledger
             where source = nominal_row.source
              and batch = nominal_row.batch  ;

            if Pnlperiod <> ThePeriod then
                ret_row.comment := '[MOVE] Batch posted to a different period';
                ret_row.source := nominal_row.source;
                ret_row.batch := nominal_row.batch;
                ret_row.subledger_total := Pslsum;
                ret_row.nominal_total := nominal_row.total;
                ret_row.nominal_period := Pnlperiod;
                return next ret_row;
            end if;
        end if;

        if Psuppress_ok <> 1 then
            ret_row.comment := 'Batch OK';
            ret_row.source := nominal_row.source;
            ret_row.batch := nominal_row.batch;
            ret_row.subledger_total := Pslsum;
            ret_row.nominal_total := nominal_row.total;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        end if;

    end loop;

   return;

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION verify_dr_nl(date, integer) OWNER TO postgres;




CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function misc20100601()
returns VOID AS
$BODY$

begin

--perform ex(' sql query here');

--perform ex(' sql query 2  in here ');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20100601() OWNER TO postgres;

-- Function: sledger_at(integer, character, date, integer)

-- DROP FUNCTION sledger_at(integer, character, date, integer);

CREATE OR REPLACE FUNCTION sledger_at(p_depot integer, p_cod character, p_date date, p_reverse integer)
  RETURNS SETOF sledger AS
$BODY$
declare sledger_row           sledger%rowtype;
        Palloc                sledger.os%type;
        Pledger_period        sledger.period%type;
        allocationNumber      integer;
	Psallocee	      sallocee%rowtype;
	Psallocer	      sallocer%rowtype;
begin

  select * from jp_period(p_date) into Pledger_period;
  Palloc := 0;

  for sledger_row in
     select *
     from sledger sl
     where sl.depot = p_depot and sl.cod = p_cod
     and sl.dat <= p_date
     order by sl.dat
  loop

     if p_reverse = 1 then
         if sledger_row.typ < 10 then
             select sum(ee.amount + ee.discount)
                 into Palloc
             from sallocee ee, sallocper per
             where ee.ee = sledger_row.ser
             and ee.allocation = per.allocation
             and per.dat > p_date;
         else
             select sum(er.amount)
                 into Palloc
             from sallocer er, sallocper per
             where er.er = sledger_row.ser
             and er.allocation = per.allocation
             and per.dat > p_date;

         end if;

         -- If the alloc amount is null set it to zero
         IF Palloc is null then
             Palloc := 0;
         end if;
     end if;

     if sledger_row.typ < 10 then
         sledger_row.os = sledger_row.os + Palloc;
     else
         sledger_row.os = sledger_row.os + Palloc;
     end if;

     -- return row if its in same period with a non zero os
     if sledger_row.os <> 0 or sledger_row.period = Pledger_period then

         return next sledger_row;

     else

	     -- return row if its involved in an allocation in the same period either allocee or allocer. Even though its os is ZERO

	     -- If its a payee

	     if sledger_row.typ < 10 then

		-- get the sallocee row

		select *
		into Psallocee
		from sallocee
		left join sallocper on ( sallocee.allocation = sallocper.allocation )
		where sallocee.ee = sledger_row.ser
		and   sallocper.period = Pledger_period;

		if NOT FOUND then
		    continue; -- if we have no sallocee
		else
		     return next sledger_row;
		end if;

	     else
		-- If its a payer

		-- get the sallocer row

		select *
		into Psallocer
		from sallocer
		left join sallocper on ( sallocer.allocation = sallocper.allocation )
		where sallocer.er = sledger_row.ser
		and   sallocper.period = Pledger_period;

		if NOT FOUND then
		    continue; -- if we have no sallocee
		else
		    return next sledger_row;
		end if;


	     end if;
	end if;
  end loop;

  return;

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION sledger_at(integer, character, date, integer) OWNER TO postgres;


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function misc20100628()
returns VOID AS
$BODY$

begin

--perform ex('');

--perform ex(' sql query 2  in here ');


perform ex('
CREATE TABLE audit_cdisposal
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer,
 lin integer,
 pdesc character(4),
 reg character(6),
 qty integer,
 unit_sell numeric(16,2),
 vcode integer,
 "location" integer,
 returned integer,
 discount numeric(4,2),
 "operator" integer,
 notes integer,
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_cdisposal OWNER TO postgres;

');


perform ex('
CREATE TABLE audit_chdetail
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer,
 lin integer,
 package_name character(4),
 package_seq integer,
 pdesc character(4) ,
 reg character(6),
 date_from date,
 date_started date,
 time_out time without time zone,
 vcode integer,
 charge_period character(1),
 date_due_back date,
 already_charged integer,
 credit integer,
 already_invoiced numeric(16,2),
 discount numeric(4,2) ,
 pricetype character(8),
 same numeric(16,2) ,
 day1 numeric(16,2) ,
 day2 numeric(16,2) ,
 day3 numeric(16,2) ,
 day4 numeric(16,2) ,
 day5 numeric(16,2) ,
 day6 numeric(16,2) ,
 week numeric(16,2) ,
 hr numeric(16,2) ,
 min_charge numeric(16,2) ,
 qty integer ,
 days_on_hire integer,
 "location" integer ,
 depot integer,
 cust character(7),
 picked integer,
 "operator" integer,
 notes integer,
 half_day numeric(16,2),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_chdetail OWNER TO postgres;

');

perform ex('
CREATE TABLE audit_chdetailext
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 "location" integer ,
 contract integer ,
 lin integer ,
 date_in timestamp without time zone,
 status integer,
 charge_line integer,
 length numeric(16,4),
 width numeric(16,4),
 height numeric(16,4),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_chdetailext OWNER TO postgres;
');

perform ex('
CREATE TABLE audit_chead
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer ,
 "location" integer ,
 cust character(7),
 depot integer,
 site integer,
 order_no character(16),
 dat date,
 tim time without time zone,
 deposit numeric(16,2),
 tender character(8),
 manual_ref integer,
 policy character(6),
 calendar character(6),
 "operator" smallint,
 invoiced_to date,
 note integer,
 hire_disc numeric(4,2) ,
 sales_disc numeric(4,2) ,
 quotation character(8),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_chead OWNER TO postgres;
');

perform ex('
CREATE TABLE audit_codetail
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer ,
 lin integer ,
 product character(7),
 unit_sell numeric(16,4),
 vcode integer,
 qty numeric(10,2) ,
 discount numeric(4,2) ,
 unit_cost numeric(16,2),
 "location" integer ,
 invoice_when character(1),
 notes integer,
 picked character(1),
 "operator" integer,
 quotation character(8),
 product_type integer,
 barcode character(13),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_codetail OWNER TO postgres;
');



end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20100628() OWNER TO postgres;

select * from misc20100628();


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function misc20100824()
returns VOID AS
$BODY$

begin

--perform ex('');

--perform ex(' sql query 2  in here ');


perform ex('
create table op_permissions(
id serial,
operator_id int ,
opt 	int ,
description text,
auth int,
primary key (id)
);
');

perform ex('alter table op_permissions add constraint fk_operator foreign key (operator_id) references operator;');


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20100824() OWNER TO postgres;

select * from misc20100824();

CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function misc20101110()
returns VOID AS
$BODY$

begin

--perform ex(' sql query in here ');


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20101110() OWNER TO postgres;

-- Function: verify_io_sum(date, date)

-- DROP FUNCTION verify_io_sum(date, date);

CREATE OR REPLACE FUNCTION verify_io_sum(pfrom date, pto date)
  RETURNS sp_status AS
$BODY$

declare err             sp_status;

        Pcod            char(6);

        Pdesc           char(30);

        Pvalue          decimal(16,2);



begin



BEGIN

    drop table t_verify_io_sum;

EXCEPTION

    when undefined_table THEN

END;



 create temp table t_verify_io_sum AS

 select a.nominal as code, d.description, sum(a.goods * c.mult) as value

    from iodetail a, ihead b, signs c, nominal d

    where b.period between Pfrom and Pto

      AND A.DOC_TYPE = B.DOC_TYpe

      AND A.location = b.location

      and a.ref = b.ref

      and a.cust = b.cust

      and a.doc_type = c.doc_type

      and a.nominal = d.cod

        group by 1,2;





    err.ret_stat := 0;

    err.ret_string := 'Success';

    return err;



end;

$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION verify_io_sum(date, date) OWNER TO postgres;


-- Function: batch_vicki_verti(character, integer)

-- DROP FUNCTION batch_vicki_verti(character, integer);

CREATE OR REPLACE FUNCTION batch_vicki_verti(psource character, pbatch integer)
  RETURNS sp_status AS
$BODY$
declare  err                     sp_status;
         nlbatch_row             nlbatch%rowtype;
         w_nxt                   integer;
         nltrans_row             nltrans%rowtype;
         auto_post               integer;
         proc_err                sp_status;
         w_rows                  integer;


begin

    select * into nlbatch_row
     from nlbatch
    where seq    = Pbatch
      and source = Psource;

    get diagnostics w_rows = ROW_COUNT;

    if w_rows = 0 then
        err.ret_stat := 1;
        err.ret_string := 'Batch ' || Psource || '/' || Pbatch || ' not on file';
        return err;
    end if;

    select nxt
      into w_nxt
      from source
     where cod = Psource;

    update source
       set nxt = nxt + 1
     where cod = Psource;

    nlbatch_row.seq := w_nxt;
    nlbatch_row.stat := 'C';

    insert into nlbatch values (nlbatch_row.source,
    nlbatch_row.seq,
    nlbatch_row.ledger,
    nlbatch_row.period,
    nlbatch_row.dat,
    nlbatch_row.location,
    nlbatch_row.stat,
    nlbatch_row.journal_ref);

    for nltrans_row in
    select *
      from nltrans
      where nltrans.source = Psource
        and nltrans.batch = Pbatch
    loop

        nltrans_row.batch = w_nxt;
        nltrans_row.tmp = 'Y';
        nltrans_row.amount = nltrans_row.amount * -1;
        nltrans_row.currency_amount = nltrans_row.currency_amount * -1;


        insert into nltrans values (default,
                                   nltrans_row.cod,
                                   nltrans_row.cc,
                                   nltrans_row.period,
                                   nltrans_row.dat,
                                   nltrans_row.source,
                                   nltrans_row.batch,
                                   nltrans_row.ref,
                                   nltrans_row.amount,
                                   nltrans_row.description,
                                   nltrans_row.notes,
                                   nltrans_row.accrual,
                                   nltrans_row.tmp,
                                   nltrans_row.location,
                                   nltrans_row.cleared,
                                   nltrans_row.currency,
                                   nltrans_row.currency_amount

        );

    end loop;

    select sys_config.set
        into auto_post
        from sys_config
        where sys_config.num = 106;

    if (not FOUND) or
        auto_post = 0 then
        err.ret_stat := 0;
        err.ret_string := 'Success';
        return err;
    else
        if auto_post = 1 then
            select * from update_batch (nlbatch_row.location, nlbatch_row.source, nlbatch_row.seq)
            into proc_err;
            return proc_err;
        end if;
    end if;



end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION batch_vicki_verti(character, integer) OWNER TO postgres;



-- Function: freeze_stock(poperator integer, pdescription bpchar, peom integer)

-- DROP FUNCTION freeze_stock(poperator integer, pdescription bpchar, peom integer);

CREATE OR REPLACE FUNCTION freeze_stock(poperator integer, pdescription bpchar, peom integer)
  RETURNS sp_status AS
$BODY$
-- Stock Freeze Header --
-- TESTED DC 2/11/2007

declare err			sp_status;
	stpt_row		record;
	Psfh_sales_period	date;

-- Stock Freeze Detail --
        Psfd_sfh_id             integer;
        Psfd_value              decimal(16,2);

begin

    err.ret_stat := 0;
    err.ret_string := 'Success';


    select period
    into Psfh_sales_period
    from dparams;

    insert into stk_freeze_hdr values 	(default,
					Poperator,
					Pdescription,
					now(),
					Psfh_sales_period,
					Peom);

    select currval('stk_freeze_hdr_nsuk_seq') into Psfd_sfh_id;

    for stpt_row in
	select b.nsuk, a.location, a.qty_physical, a.qty_onhire, b.plu, b.curr_cost_price
	from stock as a
	join product_type as b on ( a.product_type = b.nsuk )

    loop
        insert into stk_freeze_dtl values (default,
                                          Psfd_sfh_id,
                                          stpt_row.nsuk,
                                          stpt_row.plu,
                                          stpt_row.location,
                                          stpt_row.qty_physical,
                                          stpt_row.qty_onhire,
                                          stpt_row.curr_cost_price
					  );

         if Peom = 1 then
                Psfd_value := stpt_row.qty_physical - stpt_row.qty_onhire;
                insert into stk_take values (default,
                                         Psfd_sfh_id,
                                         stpt_row.plu,
                                         Psfd_value,
                                         now(),
                                         stpt_row.nsuk
					);

         end if;



    end loop;

    return err;

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION freeze_stock(poperator integer, pdescription bpchar, peom integer) OWNER TO postgres;


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function misc20110126()
returns VOID AS
$BODY$

begin

perform ex('

CREATE TABLE cheadext
(
 "location" integer NOT NULL,
 contract integer NOT NULL,
 customer_contact integer,
 internal_note integer,
 CONSTRAINT cheadext_idx PRIMARY KEY (contract, "location"),
 CONSTRAINT cheadext_fk FOREIGN KEY ("location", contract)
     REFERENCES chead ("location", contract) MATCH SIMPLE
     ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITHOUT OIDS;
');

perform ex('
ALTER TABLE cheadext OWNER TO postgres;
');

perform ex('
CREATE TABLE sig_cap_head
(
   id serial,
   depot integer,
   cust text,
   doc_number integer,
   doc_type integer,
   filename text,
   CONSTRAINT sigcapture_pk PRIMARY KEY (id)
) WITHOUT OIDS;
');

perform ex('
ALTER TABLE sig_cap_head OWNER TO postgres;
');

perform ex('
CREATE INDEX pt_supp_x4
  ON pt_supplier
  USING btree
  (supplier, supplier_ref);
');


perform ex('
CREATE TABLE ws_job_chead
(
  id serial NOT NULL,
  ws_job integer,
  "location" integer,
  contract integer,
  CONSTRAINT wsjobchead_pk PRIMARY KEY (id)
);
');

perform ex('
CREATE TABLE ws_job_ihead
(
  id serial NOT NULL,
  job_id integer,
  ihead_id integer,
  CONSTRAINT wsjobihead_pk PRIMARY KEY (id)
); 
');

perform ex('
CREATE TABLE ws_job_quotation
(
  id serial NOT NULL,
  job_id integer,
  quotation_id integer,
  CONSTRAINT wsjobquote_pk PRIMARY KEY (id)
); 
');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20110126() OWNER TO postgres;


select * from misc20110126();


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function misc20110322()
returns VOID AS
$BODY$

begin

perform ex('
create table supp_intrastat (
  id serial NOT NULL,
  cod character(6) NOT NULL,
  comm_country_code character(2),
  CONSTRAINT supp_intrastat_pkey PRIMARY KEY (id),
  CONSTRAINT supp_intrastat_fk FOREIGN KEY (cod)
      REFERENCES supplier (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION);
');

perform ex('
CREATE INDEX supp_intra_x2
  ON supp_intrastat
  USING btree
  (cod);
');

perform ex('
create table pt_intrastat (
  id serial NOT NULL,
  pt_id integer NOT NULL,
  weight numeric(16,2),
  comm_code character(10),
  CONSTRAINT pt_intrastat_pkey PRIMARY KEY (id),
  CONSTRAINT pt_intrastat_fk FOREIGN KEY (pt_id)
      REFERENCES product_type (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION);
');

perform ex('
CREATE INDEX pt_intra_x3
  ON pt_intrastat
  USING btree
  (pt_id);
');

perform ex('
create table supp_analysis (
  id serial NOT NULL,
  cod character(6) NOT NULL,
  analysis_1 character(20),
  analysis_2 character(20),
  analysis_3 character(20),
  CONSTRAINT supp_analysis_pkey PRIMARY KEY (id),
  CONSTRAINT supp_analysis_fk FOREIGN KEY (cod)
      REFERENCES supplier (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION);
');

perform ex('
CREATE INDEX supp_analysis_x2
  ON supp_analysis
  USING btree
  (cod);
');


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20110322() OWNER TO postgres;


select * from misc20110322();


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function generic2008()
returns VOID AS
$BODY$

begin

perform ex(' delete from sys_config where num = 194 ');
perform ex(' insert into sys_config values (194,''New How Many Days'',1,null,0,1) ');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION generic2008() OWNER TO postgres;
select * from generic2008();

-- Function: asatdate(asat date)

-- DROP FUNCTION asatdate(asat date);

CREATE OR REPLACE FUNCTION asatdate(asat date)
 RETURNS sp_status AS
$BODY$
declare     unmatched_row       unmatched%rowtype;
           pcost_row           record;
           Psumunmatched      integer;
           Psumpcost           integer;
           firstline            integer;
            cur_asset_reg char(1);
           cur_pdesc   char(4);
           cur_reg char(6);
           cur_qty integer;
           cur_dat date;
           cur_qty_unmatched integer;
           cur_qty_matched integer;
           cur_qty_sold integer;
           cur_sales_period date;
           cur_nominal_period date;
           cur_unit_cost decimal(16,2);
           cur_unit_depn decimal(16,2);
           cur_supp char(6);
           cur_invoice_ref char(12);
           cur_location integer;
           cur_pcost_ser integer;
           cur_qty_invoiced integer;
           err            sp_status;


begin


drop table if exists temp_unmatched;
drop table if exists temp_pcost;

create temp table temp_unmatched
 (
     ser serial not null ,
     asset_reg char(1),
     pdesc char(4),
     reg char(6),
     qty integer,
     dat date,
     cost_per_unit decimal(9,2),
     supplier char(6),
     qty_left integer,
     qty_sold integer,
     location smallint
       );

create temp table temp_pcost
    (
     ser integer,
     asset_reg char(1),
     pdesc char(4),
     reg char(6),
     unit_cost decimal(16,2),
     unit_depn decimal(16,2),
     qty integer,
     dat date,
     supplier char(6),
     invoice_ref char(12),
     source_type char(1),
     original_qty integer,
     qty_invoiced integer
      );



   cur_qty_matched :=0;
   firstline := 1;

   for pcost_row in
       select h.pcost as pcost, h.ser, h.asset_reg, h.pdesc, h.reg,
                  h.typ, h.sales_period,
                  h.dat, h.unit_cost, h.unit_depn, h.qty,
                  h.supp, h.invoice_ref, h.location, pd.typ as pdtyp

       from phistory h, pdesc pd
       where h.sales_period <= asat
       and h.pdesc = pd.cod
       and h.pcost is not null
       order by 1,2
   loop

       if firstline = 1 then
          cur_pcost_ser     := pcost_row.pcost;
          cur_asset_reg     := pcost_row.asset_reg;
          cur_pdesc         := pcost_row.pdesc;
          cur_reg           := pcost_row.reg;
          cur_qty           := 0;
          cur_qty_unmatched := 0;
          cur_qty_matched   := 0;
          cur_qty_sold      := 0;
          cur_unit_cost     := 0;
          cur_unit_depn     := 0;
          cur_invoice_ref   := '';
          cur_supp          := '';
          cur_location      := 1;
          cur_qty_invoiced  := 0;
          firstline         := 0;
       end if;

      if cur_pcost_ser <> pcost_row.pcost then

          if cur_qty_matched > 0 then

              execute 'insert into temp_pcost values (' || coalesce(quote_literal(cur_pcost_ser),'NULL') || ','
                                           || coalesce(quote_literal(cur_asset_reg),'NULL') || ','
                                           || coalesce(quote_literal(cur_pdesc),'NULL')     ||','
                                           || coalesce(quote_literal(cur_reg),'NULL')        ||','
                                           || coalesce(quote_literal(cur_unit_cost),'NULL') ||','
                                           || coalesce(quote_literal(cur_unit_depn),'NULL') ||','
                                           || coalesce(quote_literal(cur_qty_matched),'NULL')       ||','
                                           || coalesce(quote_literal(cur_dat),'NULL')       ||','
                                           || coalesce(quote_literal(cur_supp),'NULL')    ||','
                                           || coalesce(quote_literal(cur_invoice_ref),'NULL') ||','
                                           || coalesce(quote_literal(''),'NULL') ||','
                                           || coalesce(quote_literal(cur_qty_matched),'NULL') ||','
                                           || coalesce(quote_literal(cur_qty_invoiced),'NULL') ||
                                           ');';
          end if;

          cur_pcost_ser     := pcost_row.pcost;
          cur_asset_reg     := pcost_row.asset_reg;
          cur_pdesc         := pcost_row.pdesc;
          cur_reg           := pcost_row.reg;
          cur_qty           := 0;
          cur_qty_unmatched := 0;
          cur_qty_matched   := 0;
          cur_qty_sold      := 0;
          cur_unit_cost     := 0;
          cur_unit_depn     := 0;
          cur_invoice_ref   := '';
          cur_supp          := '';
          cur_location      := 1;
          cur_qty_invoiced  := 0;


       end if;

      if pcost_row.typ = 1 then
            cur_dat           := pcost_row.dat;
            cur_invoice_ref   := pcost_row.invoice_ref;
            cur_supp          := pcost_row.supp;
            cur_location      := pcost_row.location;
            cur_unit_cost     := pcost_row.unit_cost;
            cur_unit_depn     := pcost_row.unit_depn;
            cur_qty_matched   := cur_qty_matched   + pcost_row.qty;
       end if;

       if pcost_row.typ = 2 then
           cur_invoice_ref   := pcost_row.invoice_ref;
           cur_supp          := pcost_row.supp;
           cur_unit_cost     := pcost_row.unit_cost;
           cur_qty_invoiced  := cur_qty_invoiced  + pcost_row.qty;
       end if;

       if pcost_row.typ = 3 then
           if pcost_row.pdtyp = 'M' then
               cur_qty_matched   := cur_qty_matched   - pcost_row.qty;
           else
               cur_qty_matched := 0;
           end if;
       end if;

       if pcost_row.typ = 4 then
           cur_unit_depn     := cur_unit_depn     + pcost_row.unit_depn;
       end if;

       if pcost_row.typ = 5 then
           if pcost_row.qty > 0 then
               cur_unit_cost := pcost_row.unit_cost;
           end if;
       end if;

       if pcost_row.typ = 6 then
           cur_qty           := cur_qty           - hst_qty;
       end if;


       if pcost_row.typ = 7 then
           cur_dat           := pcost_row.dat;
           cur_invoice_ref   := pcost_row.invoice_ref;
           cur_supp          := pcost_row.supp;
           cur_location      := pcost_row.location;
           cur_unit_cost     := pcost_row.unit_cost;
           cur_unit_depn     := pcost_row.unit_depn;
           cur_qty_matched   := cur_qty_matched   + pcost_row.qty;
       end if;

       if pcost_row.typ = 8 then
           if pcost_row.pdtyp = 'M' then
               cur_qty_matched   := cur_qty_matched   - pcost_row.qty;
           else
               cur_qty_matched   = 0;
           end if;
       end if;

       if pcost_row.typ = 9 then
           cur_unit_depn     := cur_unit_depn     + pcost_row.unit_depn;
       end if;

   if pcost_row.typ = 10 then
           cur_qty_invoiced  := cur_qty_invoiced  - pcost_row.qty;
       end if;





   end loop;


   if cur_qty_matched <> 0 then
              execute 'insert into temp_pcost values (' || coalesce(quote_literal(cur_pcost_ser),'NULL') || ','
                                           || coalesce(quote_literal(cur_asset_reg),'NULL') || ','
                                           || coalesce(quote_literal(cur_pdesc),'NULL')     ||','
                                           || coalesce(quote_literal(cur_reg),'NULL')        ||','
                                           || coalesce(quote_literal(cur_unit_cost),'NULL') ||','
                                           || coalesce(quote_literal(cur_unit_depn),'NULL') ||','
                                           || coalesce(quote_literal(cur_qty_matched),'NULL')       ||','
                                           || coalesce(quote_literal(cur_dat),'NULL')       ||','
                                           || coalesce(quote_literal(cur_supp),'NULL')    ||','
                                           || coalesce(quote_literal(cur_invoice_ref),'NULL') ||','
                                           || coalesce(quote_literal(''),'NULL') ||','
                                           || coalesce(quote_literal(cur_qty_matched),'NULL') ||','
                                           || coalesce(quote_literal(cur_qty_invoiced),'NULL') ||
                                           ');';
   end if;

   execute 'create unique index t_ix312_1 on temp_pcost (ser)';
   execute 'create index t_multidx on temp_pcost (pdesc);';
   execute 'create index pcix2 on temp_pcost (asset_reg,pdesc,reg)';





   err.ret_stat := 0;
   err.ret_string := 'SUCCESS';

   return err;


end;
$BODY$
 LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION asatdate(asat date) OWNER TO postgres;

-- Function: getpluprices(aproducttype integer, apricelist integer, adiscstruct integer)

-- DROP FUNCTION getpluprices(aproducttype integer, apricelist integer, adiscstruct integer);

CREATE OR REPLACE FUNCTION getpluprices(aproducttype integer, apricelist integer, adiscstruct integer)
  RETURNS jp_pluprices AS
$BODY$
/*
--Kevin : 6th March 2003

This procedure returns basic product_type details, plus the Master,
Promotional and Customers prices where and if appropriate.

MASTER -->      Standard price of a product type, with Discount
                if Discount Structure specified.
PROMOTIONAL --> If the item is set on a promotional price.
                Get the Promotional Price, NO DISCOUNTs are applicable
                when dealing with Promotional prices.
CUSTOMER'S -->  If Price Specified, check for pricelist value, if none
                exists use MASTER price.
                If Discount Structure specified apply it to value.
*/

-- Internal Variables

declare pPLU                             text;
       pDescription                     char(90);
       pProdCode                        text;
       pProdDescription                 char(90);
       pCostPrice                       decimal(16,2);
       pVatCode                         integer;
       pVatRate                         decimal(4,2);
       pMasterSP                        decimal(16,4);
       pCustomerSP                      decimal(16,4);
       pPromoSP                         decimal(16,4);
       pSellPrice                       decimal(16,4);
       pDiscountRate                    decimal(4,2);
       pDiscount                        decimal(16,2);
       pDiscountable                    smallint;
       pLookForDisc                     smallint;
       pCheckPLDisc                     smallint;

       pProduct                         integer;   -- Product Nsuk
       pStockUnit                       char(10);  -- Product Stock Unit
       pRowCnt                          smallint;  -- No. of rows returned
       pPLNsuk                          integer;   -- Holds PriceList Nsuk
       pPromoNsuk                       integer;   -- Holds Promotion Nsuk
       pDSNsuk                          integer;   -- Holds DiscStruct Nsuk
       pDept                            integer;   -- Department
       pDeptGroup                       integer;   -- Department Group
       pPLparent                        integer;   -- Parent price list
       pSellPriceType                   smallint;  -- Sell Price Type
       ret_price             jp_pluprices;

begin


  pMasterSP := null;
  pCustomerSP := null;
  pPromoSP := null;
  pSellPrice := 0.00;
  pDiscountRate := null;
  pDiscount := 0.00;
  pSellPriceType := 0;
  pPromoNsuk := 0;
  pStockUnit := '';

  pLookForDisc := 1;
  pCheckPLDisc := 0;

/* Intialise the jp_pluprices type to what it will be if we have to return an error
*/

  ret_price.pt := 0;
  ret_price.plu := '';
  ret_price.plu_description := '';
  ret_price.product_code := '';
  ret_price.product_description := '';
  ret_price.cost_price := 0;
  ret_price.vat_code := 0;
  ret_price.vat_rate := 0;
  ret_price.master_price := 0;
  ret_price.promotional_price := 0;
  ret_price.customer_price := 0;
  ret_price.sell_price := 0;
  ret_price.discount_rate := 0;
  ret_price.discount_value := 0;
  ret_price.sell_pricetype := 0;
  ret_price.stock_unit := '';

-- Step One : Get product_type details

       SELECT  plu, product, descr, curr_cost_price, curr_sell_price
       INTO    pPLU, pProduct, pDescription, pCostPrice, pMasterSP
       FROM    product_type
       WHERE   nsuk = aProductType;

   get diagnostics pRowCnt = ROW_COUNT;

       if pRowCnt > 1 then
               ret_price.err_code = 1;
               ret_price.err_text = 'More than one rows returned for PLU';
               return ret_price;
       end if;

       if pRowCnt = 0 then
               ret_price.err_code = 1;
               ret_price.err_text = 'No rows returned for PLU';
               return ret_price;
       end if;

-- Step 2 : Get Vat Rate and Code.
    -- Get VatCode from Product
    -- Get VatRate from Vat

    SELECT p.vcode, v.rate, p.cod, p.description, p.stock_unit
    INTO   pVatCode, pVatRate, pProdCode, pProdDescription, pStockUnit
    FROM   product p, vat v
    WHERE  p.nsuk = pProduct
      AND  p.vcode = v.cod;

    if pStockUnit is null then
      pStockUnit := '';
    end if;


-- Step 3 : Get Dept and Dept_Group Details

   SELECT dg.dept, dg.nsuk
   INTO pDept, pDeptGroup
   FROM dept_group dg , product p
   WHERE p.nsuk = pProduct
     AND p.dept_group = dg.nsuk;


-- Step 4 : Check if pricelist is Discountable or not

   if aPriceList <> 0 then
     SELECT parent
     INTO pPLparent
     FROM pricelist
     WHERE nsuk = aPricelist;

     if pPLparent is null then
       pPLparent := 0;
     end if;

   end if;

   pDiscountable := 1;


-- Step 5 : Check see if a Promotional Price exists for this item.
    -- Get NSUK for PROMOTION price list (if it exists).

    SELECT nsuk
    INTO pPromoNsuk
    FROM pricelist
    WHERE cod = 'PROMOTION';

    -- If we Found PROMOTION pricelist

    get diagnostics pRowCnt = ROW_COUNT;
    if pRowCnt = 1 then

       -- Check for PLU on Promotion Price List

       SELECT sell_price
       INTO   pPromoSP
       FROM pricelist_det
       WHERE pricelist = pPromoNsuk
         AND product_type = aProductType;

       if not FOUND then

           SELECT sell_price
           INTO   pPromoSP
           FROM pricelist_det
           WHERE pricelist = pPromoNsuk
             AND product = pProduct;

           if not FOUND then
              pPromoSP = null;
           else
              pLookForDisc = 0;    -- Don't Look for Discount
           end if;
       else
           pLookForDisc = 0;        -- Don't Look for Discount
       end if;

    end if;

-- Step 6 : Check see if a Customer Price exists for this item.

    -- Step 6.1 : If Customer gave a Price List, Check list for Price

    if aPriceList > 0 then

         SELECT sell_price
         INTO   pCustomerSP
         FROM pricelist_det
         WHERE pricelist = aPriceList
           AND product_type = aProductType;

         if not FOUND then

             SELECT sell_price
             INTO   pCustomerSP
             FROM pricelist_det
             WHERE pricelist = aPriceList
               AND product = pProduct;

             if not FOUND then

                 if pPLparent <> 0 then
                     SELECT sell_price
                     INTO   pCustomerSP
                     FROM pricelist_det
                     WHERE pricelist = pPLparent
                       AND product_type = aProductType;

                     if not FOUND then

                         SELECT sell_price
                         INTO   pCustomerSP
                         FROM pricelist_det
                         WHERE pricelist = pPLparent
                           AND product = pProduct;

                         if not FOUND then
                            pCustomerSP = null;
                         else
                            pCheckPLDisc = 1;        --  Look for Discount
                         end if;
                     else
                         pCheckPLDisc = 1;        --  Look for Discount
                     end if;
                 else
                    -- there is no parent price list
                    pCustomerSP = null;
                 end if;
             else
                 pCheckPLDisc = 1;
             end if;
         else
             pCheckPLDisc = 1;        --  Look for Discount
         end if;

         pDiscountable := null;

         if pCheckPLDisc = 1 then
            SELECT discountable
            INTO pDiscountable
            FROM pricelist
            WHERE nsuk = aPricelist;

            if pDiscountable is null then
              pDiscountable := 0;
            end if;

            if pDiscountable = 0 then
              pLookForDisc := 0;
            end if;
         end if;

    end if;

-- Step 7: Get Discount Structure Details, IF ANY

  if aDiscStruct <> 0 then

    if pLookForDisc > 0 then

       -- First Check for Discount price for DEPT and DEPT GROUP
       SELECT distinct(discount)
       INTO pDiscountRate
       FROM disc_detail
       WHERE disc_struct = aDiscStruct
         -- AND department = pDept
         AND dept_group = pDeptGroup;

      get diagnostics pRowCnt = ROW_COUNT;
      -- If No results for Dept and Dept Group
      if pRowCnt <> 1 then

          -- Then Check for Discount for just Dept

          SELECT distinct(discount)
          INTO pDiscountRate
          FROM disc_detail
          WHERE disc_struct = aDiscStruct
            AND department = pDept
            AND (dept_group is null or dept_group = -1);

          get diagnostics pRowCnt = ROW_COUNT;
          -- If No Results then Discount is Zero
          if pRowCnt <> 1 then
             pDiscountRate := null;
          end if;

      end if;

    else
       pDiscountRate := null;
    end if; -- if pDiscountable > 0

  end if; -- if aDiscStruct <> ''


-- Step 8: Work out SELL_PRICE to use.

    -- If we have a promotional Price USE THAT
    if pPromoSP is not null then
       pSellPrice := pPromoSP;
       pSellPriceType := 1;
    else
       -- NOT promo price so see if Customer or Standard price to be used

       if pCustomerSP is not null then

          -- if Theres a Discount, apply it
          if pDiscountRate is not null then
             pDiscount := ( pCustomerSP / 100 ) * pDiscountRate;
          else
             pDiscount := null;
          end if;

          pSellPrice := pCustomerSP;
          pSellPriceType := 2;
       else

          -- if Theres a Discount, apply it
          if pDiscountRate is not null then
             pDiscount := ( pMasterSP / 100 ) * pDiscountRate;
          else
             pDiscount := null;
          end if;

          pSellPrice := pMasterSP;
          pSellPriceType := 3;
       end if;

    end if;

-- Step 9 : Return Findings

  ret_price.err_code := 0;
  ret_price.err_text := 'Success';
  ret_price.pt := aProductType;
  ret_price.plu := pPLu;
  ret_price.plu_description := pDescription;
  ret_price.product_code := pProdCode;
  ret_price.product_description := pProdDescription;
  ret_price.cost_price := pCostPrice;
  ret_price.vat_code := pVatCode;
  ret_price.vat_rate := pVatRate;
  ret_price.master_price := pMasterSP;
  ret_price.promotional_price := pPromoSP;
  ret_price.customer_price := pCustomerSP;
  ret_price.sell_price := pSellPrice;
  ret_price.discount_rate := pDiscountRate;
  ret_price.discount_value := pDiscount;
  ret_price.sell_pricetype := pSellPriceType;
  ret_price.stock_unit := pStockUnit;

  return ret_price;

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION getpluprices(aproducttype integer, apricelist integer, adiscstruct integer) OWNER TO postgres;

create or replace function generic2010()
returns VOID AS
$BODY$

begin

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION generic2010() OWNER TO postgres;


CREATE OR REPLACE VIEW v_supplier_p_sanalc AS
 SELECT x0.ac_location, p2.supplier, p3.supplier_ref AS plantorprod, x1.product_type, x0.period,

       CASE
           WHEN x0.doc_type = 'CR'::bpchar THEN x1.goods * (- 1::numeric)
           ELSE x1.goods
       END AS amount,

       CASE
       WHEN x0.doc_type = 'CR'::bpchar THEN x1.cost * x1.qty * (- 1::numeric)
           ELSE x1.cost * x1.qty
       END
       AS cost ,


       CASE
           WHEN x0.doc_type = 'CR'::bpchar THEN x1.qty * (- 1::numeric)
           ELSE x1.qty
       END AS qty


 FROM ihead x0, iodetail x1, product_type p1, product p2, pt_supplier p3
 WHERE x0.doc_type = x1.doc_type AND x0."location" = x1."location"
 AND x0.ref = x1.ref AND x0.cust = x1.cust and x1.product_type = p1.nsuk
 and p1.product = p2.nsuk and p3.product_type = p1.nsuk and p3.supplier = p2.supplier;
 --GROUP BY x0.ac_location, p2.supplier, p3.supplier_ref, x1.product_type, x0.period;

ALTER TABLE v_supplier_p_sanalc  OWNER TO postgres;

create or replace function generic2011()
returns VOID AS
$BODY$

begin

perform ex('
CREATE TABLE pdesc_safety_msg
(
  id serial NOT NULL,
  pdesc_id character(4),
  message text,
  CONSTRAINT pdesc_safety_msg_pkey PRIMARY KEY (id),
  CONSTRAINT "FK_pdesc_safety_msg" FOREIGN KEY (pdesc_id)
      REFERENCES pdesc (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITHOUT OIDS;
');

perform ex('
ALTER TABLE pdesc_safety_msg OWNER TO postgres;
');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION generic2011() OWNER TO postgres;

select * from generic2011();

create or replace function generic2012()
returns VOID AS
$BODY$

begin

perform ex(' insert into plant_status values (10,''Plant Code Change''); ');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION generic2012() OWNER TO postgres;

select * from generic2012();


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function generic2013()
returns VOID AS
$BODY$

begin

perform ex('

    create table category_tree(
    id serial,
    parent integer,
    descr text
    );

');

perform ex(' alter table category_tree add primary key (id); ');
perform ex(' insert into category_tree values ( 1,1,''Categories''); ');

perform ex('

    create table brand(
    id serial,
    cat_tree_id integer,
    name text,
    url text
    );

');

perform ex(' alter table brand add primary key (id); ');
perform ex(' alter table brand add constraint cat_tree_fk foreign key (cat_tree_id) references category_tree(id); ');

perform ex('

    create table pt_web_category(
    id serial,
    product_type_id integer,
    web_cat_id integer
    );

');


perform ex(' alter table pt_web_category add primary key (id); ');
perform ex(' alter table pt_web_category add constraint pt_web_cat_pt_fk foreign key (product_type_id) references product_type(nsuk); ');
perform ex(' alter table pt_web_category add constraint pt_web_cat_cat_fk foreign key (web_cat_id) references category_tree(id); ');

perform ex('

    create table pt_seo_details(
    id serial,
    product_type_id integer,
    page_title text,
    meta_descr text,
    seo_keywords_id integer
    );

');


perform ex(' alter table pt_seo_details add primary key (id); ');
perform ex(' alter table pt_seo_details add constraint pt_seo_pt_fk foreign key (product_type_id) references product_type(nsuk); ');

perform ex('

    create table pt_seo_keywords(
    id serial,
    pt_seo_det_id integer,
    keyword text
    );

');

perform ex(' alter table pt_seo_keywords add primary key (id); ');
perform ex(' alter table pt_seo_keywords add constraint pt_seo_key_fk foreign key (pt_seo_det_id) references pt_seo_details(id); ');

perform ex('

    create table pt_dimension(
    id serial,
    product_type_id integer,
    length	decimal(10,2),
    width	decimal(10,2),
    height	decimal(10,2),
    weight	decimal(10,2)
    );

');

perform ex(' alter table pt_dimension add primary key (id); ');
perform ex(' alter table pt_dimension add constraint pt_dim_pt_fk foreign key (product_type_id) references product_type(nsuk); ');


perform ex('

    create table image_type(
    id serial,
    typ integer,
    description text
    );

');


perform ex(' alter table image_type add primary key (id); ');
perform ex(' insert into image_type values (1,1,''Default''); ');
perform ex(' insert into image_type values (2,2,''Normal''); ');


perform ex('

    create table pt_web_image(
    id serial,
    product_type_id integer,
    image_url text,
    image_type_id integer
    );

');


perform ex(' alter table pt_web_image add primary key (id); ');
perform ex(' alter table pt_web_image add constraint pt_web_img_fk foreign key (image_type_id) references image_type(id); ');
perform ex(' alter table pt_web_image add constraint pt_webimg_pt_fk foreign key (product_type_id) references product_type(nsuk); ');


perform ex('

    create table pt_listing(
    id serial,
    descr text
    );

');


perform ex(' alter table pt_listing add primary key(id); ');
perform ex(' insert into pt_listing values(1,''Web And Local''); ');
perform ex(' insert into pt_listing values(2,''Web Only''); ');
perform ex(' insert into pt_listing values(3,''Local Only''); ');


perform ex('

    create table pt_web_detail(
    id serial,
    product_type_id integer,
    product_title text,
    description_long text,
    brand_id integer,
    pricelist_id integer,
    pt_listing_id integer,
    live_on_web boolean,
    pt_transport_id integer
    );

');

perform ex(' alter table pt_web_detail add primary key (id); ');
perform ex(' alter table pt_web_detail add constraint pt_web_det_pt_fk foreign key (product_type_id) references product_type(nsuk); ');
perform ex(' alter table pt_web_detail add constraint pt_web_det_br_fk foreign key (brand_id) references brand(id); ');
perform ex(' alter table pt_web_detail add constraint pt_web_det_pr_fk foreign key (pricelist_id) references pricelist(nsuk); ');
perform ex(' alter table pt_web_detail add constraint pt_web_list_fk foreign key (pt_listing_id) references pt_listing(id); ');



perform ex('

    create table pt_transport(
    id serial,
    product_type_id integer
    );

');


perform ex(' alter table pt_transport add primary key(id); ');
perform ex(' alter table pt_transport add constraint pt_transport_fk foreign key(product_type_id) references product_type(nsuk); ');



perform ex(' select * from sys_refresh_sequences(); ');



end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION generic2013() OWNER TO postgres;
select * from generic2013();


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function generic2014()
returns VOID AS
$BODY$

begin

perform ex('
    create table ihead_chead_sled (
    id serial,
    ihead_id integer,
    chead_location integer,
    chead_contract integer,
    sledger_ser    integer,
    constraint ihead_chead_sled_pkey primary key( id )
    );
');

perform ex(' create index cheadbridge on ihead_chead_sled ( chead_location,chead_contract ); ');
perform ex(' create index cheadihead on ihead_chead_sled ( ihead_id ); ');
perform ex(' create index iheadsled on ihead_chead_sled ( sledger_ser ); ');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION generic2014() OWNER TO postgres;

select * from generic2014();


CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function generic2015()
returns VOID AS
$BODY$

begin

perform ex(' insert into plant_status values (11,''Not Ready For Hire''); ');


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION generic2015() OWNER TO postgres;

select * from generic2015();


create or replace function databasebuildtable()
returns VOID AS
$BODY$
begin
perform ex('
create table databasebuild (
                    id serial,
                    gen_build_num int,
                    ver_build_num int,
                    date_run date,
	        primary key(id)
)  
');
	perform ex('
		delete from databasebuild;
            ');

	perform ex('
		insert into databasebuild values (1,2015,2040,null);
            ');

end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION databasebuildtable() OWNER TO postgres;

select * from databasebuildtable();