﻿CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$
begin
	BEGIN
		execute query;
	EXCEPTION
		WHEN duplicate_table THEN
	             --do nothing we want to run all the way through;
        WHEN duplicate_object THEN
                 --do nothing we want to run all the way through;
        WHEN invalid_table_definition THEN
                 --do nothing we want to run all the way through;
        WHEN invalid_foreign_key THEN
                 --do nothing we want to run all the way through;
        WHEN undefined_table THEN
		 --do nothing we want to run all the way through;
	END;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


create or replace function misctables()
returns VOID AS
$BODY$
begin

perform ex(E'

CREATE TABLE nominal_lookup
(
  nsuk serial NOT NULL,
  external_code character(20),
  nominal_code character(6),
  primary key (nsuk)
);

');

perform ex(E'
CREATE TABLE vat_lookup
(
  nsuk serial NOT NULL,
  vcode smallint,
  external_code integer,
  primary key (nsuk)
);

');

perform ex(E'
CREATE TABLE non_disc_item
(
    nsuk serial NOT NULL,
    pdesc_asset_reg character(1),
    pdesc_id character(4),
    product_id integer,
    product_type_id integer,
    primary key (nsuk)
);
');



perform ex(E'
CREATE TABLE audit_cdisposal
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer,
 lin integer,
 pdesc character(4),
 reg character(6),
 qty integer,
 unit_sell numeric(16,2),
 vcode integer,
 "location" integer,
 returned integer,
 discount numeric(4,2),
 "operator" integer,
 notes integer,
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_cdisposal OWNER TO postgres;

');


perform ex(E'
CREATE TABLE audit_chdetail
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer,
 lin integer,
 package_name character(4),
 package_seq integer,
 pdesc character(4) ,
 reg character(6),
 date_from date,
 date_started date,
 time_out time without time zone,
 vcode integer,
 charge_period character(1),
 date_due_back date,
 already_charged integer,
 credit integer,
 already_invoiced numeric(16,2),
 discount numeric(4,2) ,
 pricetype character(8),
 same numeric(16,2) ,
 day1 numeric(16,2) ,
 day2 numeric(16,2) ,
 day3 numeric(16,2) ,
 day4 numeric(16,2) ,
 day5 numeric(16,2) ,
 day6 numeric(16,2) ,
 week numeric(16,2) ,
 hr numeric(16,2) ,
 min_charge numeric(16,2) ,
 qty integer ,
 days_on_hire integer,
 "location" integer ,
 depot integer,
 cust character(7),
 picked integer,
 "operator" integer,
 notes integer,
 half_day numeric(16,2),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_chdetail OWNER TO postgres;

');

perform ex(E'
CREATE TABLE audit_chdetailext
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 "location" integer ,
 contract integer ,
 lin integer ,
 date_in timestamp without time zone,
 status integer,
 charge_line integer,
 length numeric(16,4),
 width numeric(16,4),
 height numeric(16,4),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_chdetailext OWNER TO postgres;
');

perform ex(E'
CREATE TABLE audit_chead
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer ,
 "location" integer ,
 cust character(7),
 depot integer,
 site integer,
 order_no character(16),
 dat date,
 tim time without time zone,
 deposit numeric(16,2),
 tender character(8),
 manual_ref integer,
 policy character(6),
 calendar character(6),
 "operator" smallint,
 invoiced_to date,
 note integer,
 hire_disc numeric(4,2) ,
 sales_disc numeric(4,2) ,
 quotation character(8),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_chead OWNER TO postgres;
');

perform ex(E'
CREATE TABLE audit_codetail
(
 audit_nsuk serial NOT NULL,
 audit_timestamp timestamp without time zone,
 audit_operator integer,
 audit_type integer,
 contract integer ,
 lin integer ,
 product character(7),
 unit_sell numeric(16,4),
 vcode integer,
 qty numeric(10,2) ,
 discount numeric(4,2) ,
 unit_cost numeric(16,2),
 "location" integer ,
 invoice_when character(1),
 notes integer,
 picked character(1),
 "operator" integer,
 quotation character(8),
 product_type integer,
 barcode character(13),
 primary key (audit_nsuk)
)
WITHOUT OIDS;
ALTER TABLE audit_codetail OWNER TO postgres;
');

perform ex(E'
create table op_permissions(
id serial,
operator_id int ,
opt 	int ,
description text,
auth int,
primary key (id)
);
');

perform ex(E'

CREATE TABLE cheadext
(
 "location" integer NOT NULL,
 contract integer NOT NULL,
 customer_contact integer,
 internal_note integer,
 CONSTRAINT cheadext_idx PRIMARY KEY (contract, "location"),
 CONSTRAINT cheadext_fk FOREIGN KEY ("location", contract)
     REFERENCES chead ("location", contract) MATCH SIMPLE
     ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITHOUT OIDS;
');

perform ex(E'
ALTER TABLE cheadext OWNER TO postgres;
');

perform ex(E'
CREATE TABLE sig_cap_head
(
   id serial,
   depot integer,
   cust text,
   doc_number integer,
   doc_type integer,
   filename text,
   CONSTRAINT sigcapture_pk PRIMARY KEY (id)
) WITHOUT OIDS;
');

perform ex(E'
ALTER TABLE sig_cap_head OWNER TO postgres;
');

perform ex(E'
CREATE TABLE ws_job_chead
(
  id serial NOT NULL,
  ws_job integer,
  "location" integer,
  contract integer,
  CONSTRAINT wsjobchead_pk PRIMARY KEY (id)
);
');

perform ex(E'
CREATE TABLE ws_job_ihead
(
  id serial NOT NULL,
  job_id integer,
  ihead_id integer,
  CONSTRAINT wsjobihead_pk PRIMARY KEY (id)
);
');

perform ex(E'
CREATE TABLE ws_job_quotation
(
  id serial NOT NULL,
  job_id integer,
  quotation_id integer,
  CONSTRAINT wsjobquote_pk PRIMARY KEY (id)
);
');

perform ex(E'
create table supp_intrastat (
  id serial NOT NULL,
  cod character(6) NOT NULL,
  comm_country_code character(2),
  CONSTRAINT supp_intrastat_pkey PRIMARY KEY (id),
  CONSTRAINT supp_intrastat_fk FOREIGN KEY (cod)
      REFERENCES supplier (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION);
');


perform ex(E'
create table pt_intrastat (
  id serial NOT NULL,
  pt_id integer NOT NULL,
  weight numeric(16,2),
  comm_code character(10),
  CONSTRAINT pt_intrastat_pkey PRIMARY KEY (id),
  CONSTRAINT pt_intrastat_fk FOREIGN KEY (pt_id)
      REFERENCES product_type (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION);
');


perform ex(E'
create table supp_analysis (
  id serial NOT NULL,
  cod character(6) NOT NULL,
  analysis_1 character(20),
  analysis_2 character(20),
  analysis_3 character(20),
  CONSTRAINT supp_analysis_pkey PRIMARY KEY (id),
  CONSTRAINT supp_analysis_fk FOREIGN KEY (cod)
      REFERENCES supplier (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION);
');


perform ex(E'
CREATE TABLE pdesc_safety_msg
(
  id serial NOT NULL,
  pdesc_id character(4),
  message text,
  CONSTRAINT pdesc_safety_msg_pkey PRIMARY KEY (id),
  CONSTRAINT "FK_pdesc_safety_msg" FOREIGN KEY (pdesc_id)
      REFERENCES pdesc (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITHOUT OIDS;
');

perform ex(E'
ALTER TABLE pdesc_safety_msg OWNER TO postgres;
');

perform ex(E'

    create table category_tree(
    id serial,
    parent integer,
    descr text
    );

');

perform ex(E'

    create table brand(
    id serial,
    cat_tree_id integer,
    name text,
    url text
    );

');

perform ex(E'

    create table pt_web_category(
    id serial,
    product_type_id integer,
    web_cat_id integer
    );

');

perform ex(E'

    create table pt_seo_details(
    id serial,
    product_type_id integer,
    page_title text,
    meta_descr text,
    seo_keywords_id integer
    );

');

perform ex(E'

    create table pt_seo_keywords(
    id serial,
    pt_seo_det_id integer,
    keyword text
    );

');

perform ex(E'

    create table pt_dimension(
    id serial,
    product_type_id integer,
    length	decimal(10,2),
    width	decimal(10,2),
    height	decimal(10,2),
    weight	decimal(10,2)
    );

');

perform ex(E'

    create table image_type(
    id serial,
    typ integer,
    description text
    );

');

perform ex(E'

    create table pt_web_image(
    id serial,
    product_type_id integer,
    image_url text,
    image_type_id integer
    );

');

perform ex(E'

    create table pt_listing(
    id serial,
    descr text
    );

');

perform ex(E'

    create table pt_web_detail(
    id serial,
    product_type_id integer,
    product_title text,
    description_long text,
    brand_id integer,
    pricelist_id integer,
    pt_listing_id integer,
    live_on_web boolean,
    pt_transport_id integer
    );

');

perform ex(E'

    create table pt_transport(
    id serial,
    product_type_id integer
    );

');

perform ex(E'
    create table ihead_chead_sled (
    id serial,
    ihead_id integer,
    chead_location integer,
    chead_contract integer,
    sledger_ser    integer,
    constraint ihead_chead_sled_pkey primary key( id )
    );
');

perform ex(E'

create table corder_ihead_map(
    id serial,
    custorder_id integer,
    ihead_id     integer,
    constraint corder_ihead_map_pk primary key( id )
    );

 ');

perform ex(E'

CREATE TABLE o_disposal
(
  nsuk serial NOT NULL,
  "order" integer NOT NULL,
  line_number integer NOT NULL,
  quotation_line integer,
  asset_reg character(1) NOT NULL,
  pdesc character(4) NOT NULL,
  reg character(5),
  unit_sell numeric(16,2),
  unit_cost numeric(16,2),
  qty integer,
  vcode integer,
  discount numeric(16,2) NOT NULL,
  invoice_when character(1) NOT NULL,
  note integer,
  "operator" integer,
  status integer
)

WITHOUT OIDS;
ALTER TABLE o_disposal OWNER TO postgres;

');


perform ex(E'
    create table operator_metadata(
    id serial,
    foreign_id int,
    attribute text,
    value text,
    primary key (id)
    );
 ');


perform ex(E'

    CREATE TABLE pdesc_residual
    (
    id serial NOT NULL,
    pdesc_id character(4),
    residual_percent numeric(16,2),
    CONSTRAINT pdesc_residual_pkey PRIMARY KEY (id)
    )
    WITHOUT OIDS;

');

perform ex(E'

    CREATE TABLE singles_residual
    (
    id serial NOT NULL,
    singles_pdesc character(4),
    singles_cod character(6),
    residual_value numeric(16,2),
    CONSTRAINT singles_residual_pkey PRIMARY KEY (id)
    )
    WITHOUT OIDS;

');


perform ex(E'

create table banknlxref
  (
    nsuk serial not null ,
    nltrans_id integer,
    bank_rec_id integer,
    cleared integer,
    primary key (nsuk)
  );

');

perform ex(E'

create table bank_rec
  (
    nsuk serial not null ,
    statement_date date,
    opening_balance decimal(16,2),
    closing_balance decimal(16,2),
    operator_id smallint,
    date_processed date,
    primary key (nsuk)
  );

');

perform ex(E'
CREATE TABLE new_nlbals
(
  cod character(6),
  bop_bal numeric,
  cur_bal numeric,
  ytd_bal numeric,
  ly_bop_bal numeric,
  ly_cur_bal numeric,
  ly_ytd_bal numeric
)

');

perform ex(E'

CREATE TABLE new_t_bopbal
(
  cod character(6),
  cc character(4),
  bop_bal numeric
)
');


perform ex(E'

CREATE TABLE new_t_bopbal_ly
(
  cod character(6),
  cc character(4),
  bop_bal numeric
)
');


perform ex(E'

CREATE TABLE new_t_curbal
(
  cod character(6),
  cc character(4),
  cur_bal numeric
)
');


perform ex(E'
CREATE TABLE new_t_curbal_ly
(
  cod character(6),
  cc character(4),
  cur_bal numeric
)
');


perform ex(E'
CREATE TABLE new_t_ytdbal
(
  cod character(6),
  cc character(4),
  ytd_bal numeric
)
');


perform ex(E'

CREATE TABLE new_t_ytdbal_ly
(
  cod character(6),
  cc character(4),
  ytd_bal numeric
)
');

perform ex(E'

    CREATE TABLE new_cust_age
    (
    period date NOT NULL,
    depot integer NOT NULL,
    cod character(7) NOT NULL,
    slave_depot integer,


    slave_cod character(7),
    balance numeric(16,2) NOT NULL,
    unallocated numeric(16,2) NOT NULL,
    account_type character(1) NOT NULL,


    suspend character(1) NOT NULL,
    bad_debt character(1) NOT NULL,
    lastpay numeric(16,2),
    lastpaid date,
    current numeric(16,2) NOT

    NULL,
    age1 numeric(16,2) NOT NULL,
    age2 numeric(16,2) NOT NULL,
    age3 numeric(16,2) NOT NULL,
    age4 numeric(16,2) NOT NULL,
    age5

    numeric(16,2) NOT NULL,
    master_slave character(1) NOT NULL,
    slave_count integer NOT NULL
    )

');


perform ex(E' create table baddebt_check (ser NUMERIC, cod CHAR(7), typ INT, depot INT, bad_debt CHAR(1))  ');

perform ex(E' create table cust_balance (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' CREATE TABLE current_age (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age1 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age2 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2)) ');
perform ex(E' create table cust_age3 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age4 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age5 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table lastpay (ser NUMERIC, COD CHAR(7), depot INT)  ');
perform ex(E' create table lastpay2 (cod CHAR(7), depot INT, dat DATE, amount NUMERIC (16,2))  ');
perform ex(E' CREATE TABLE sun_unalloc (amount NUMERIC (16,2), cod CHAR(7), depot INT)  ');



perform ex(E'

    CREATE TABLE new_cust_age
    (
    period date NOT NULL,
    depot integer NOT NULL,
    cod character(7) NOT NULL,
    slave_depot integer,


    slave_cod character(7),
    balance numeric(16,2) NOT NULL,
    unallocated numeric(16,2) NOT NULL,
    account_type character(1) NOT NULL,


    suspend character(1) NOT NULL,
    bad_debt character(1) NOT NULL,
    lastpay numeric(16,2),
    lastpaid date,
    current numeric(16,2) NOT

    NULL,
    age1 numeric(16,2) NOT NULL,
    age2 numeric(16,2) NOT NULL,
    age3 numeric(16,2) NOT NULL,
    age4 numeric(16,2) NOT NULL,
    age5

    numeric(16,2) NOT NULL,
    master_slave character(1) NOT NULL,
    slave_count integer NOT NULL
    )

');


perform ex(E' create table baddebt_check (ser NUMERIC, cod CHAR(7), typ INT, depot INT, bad_debt CHAR(1))  ');

perform ex(E' create table cust_balance (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' CREATE TABLE current_age (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age1 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age2 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2)) ');
perform ex(E' create table cust_age3 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age4 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table cust_age5 (depot INT, cod CHAR(7), amount NUMERIC (16,2), cur_bal NUMERIC (16,2))  ');
perform ex(E' create table lastpay (ser NUMERIC, COD CHAR(7), depot INT)  ');
perform ex(E' create table lastpay2 (cod CHAR(7), depot INT, dat DATE, amount NUMERIC (16,2))  ');
perform ex(E' CREATE TABLE sun_unalloc (amount NUMERIC (16,2), cod CHAR(7), depot INT)  ');

perform ex(E' DROP VIEW v_jlproof ');

perform ex(E' alter table nltrans alter column description type text ');

perform ex(E'
CREATE OR REPLACE VIEW v_jlproof AS
 SELECT x2.source, x2.batch, x0.nominal_period AS period, x0.dat, x0.note AS jrnl_note, x1.ledger, x1.depot, x1.cod, x2.description, x2.cc, x2.amount, x2.notes AS note
   FROM journals x0, jdetails x1, nltrans x2
  WHERE x0.ser = x1.journal_no AND x1.nl_trans = x2.trans_no;
');

perform ex(E' ALTER TABLE v_jlproof OWNER TO postgres ');


perform ex(E' DROP VIEW v_turnover ');

perform ex(E' alter table supplier alter column nam type text ');

perform ex(E'
CREATE OR REPLACE VIEW v_turnover AS
 SELECT pl.supplier, supp.nam, pl.period, sum(pl.amount - pl.vat) AS amount
   FROM pledger pl, supplier supp
  WHERE (pl.typ = 2 OR pl.typ = 11) AND supp.cod = pl.supplier
  GROUP BY pl.supplier, supp.nam, pl.period;
');

perform ex(E' ALTER TABLE v_turnover OWNER TO postgres ');

perform ex ('alter table o_sale alter column qty type decimal(16,2) ');
perform ex ('alter table q_sale alter column qty type decimal(16,2) ');

perform ex(E'
CREATE TABLE supp_bank_ext
(
  id serial NOT NULL,
  cod character(6) NOT NULL,
  iban text,
  swift_code text,
  CONSTRAINT supp_bank_ext_pkey PRIMARY KEY (id ),
  CONSTRAINT supp_bank_ext_fk FOREIGN KEY (cod)
      REFERENCES supplier (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
');

perform ex(E' ALTER TABLE supp_bank_ext OWNER TO postgres ');

perform ex ('alter table o_sale alter column qty type decimal(16,2) ');
perform ex ('alter table q_sale alter column qty type decimal(16,2) ');

perform ex(E'
CREATE TABLE supp_bank_ext
(
  id serial NOT NULL,
  cod character(6) NOT NULL,
  iban text,
  swift_code text,
  CONSTRAINT supp_bank_ext_pkey PRIMARY KEY (id ),
  CONSTRAINT supp_bank_ext_fk FOREIGN KEY (cod)
      REFERENCES supplier (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
');

perform ex(E' ALTER TABLE supp_bank_ext OWNER TO postgres ');


perform ex ('alter table o_sale alter column qty type decimal(16,2) ');
perform ex ('alter table q_sale alter column qty type decimal(16,2) ');

perform ex(E'
CREATE TABLE supp_bank_ext
(
  id serial NOT NULL,
  cod character(6) NOT NULL,
  iban text,
  swift_code text,
  CONSTRAINT supp_bank_ext_pkey PRIMARY KEY (id ),
  CONSTRAINT supp_bank_ext_fk FOREIGN KEY (cod)
      REFERENCES supplier (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
)
');

perform ex(E' ALTER TABLE supp_bank_ext OWNER TO postgres ');

perform ex(E'
    create table iod_ext
    (
    id serial,
    iodetail_id int,
    list_price numeric(16,4)
    ) ;
');

perform ex(E'
    create table loyalty_member(
    id serial,
    depot integer not null,
    cod character(7),
    l_date_start date,
    points_redeemed integer,
    enabled boolean,
    primary key(id)
    );
');

perform ex(E'
    create table loyalty_rewards(
    id serial,
    points integer,
    reward_amount numeric(16,2),
    discount_perc numeric(6,2),
    description text,
    primary key(id)
    );
');

perform ex(E'
    create table loyalty_threshold(
    id serial,
    threshold_level int,
    points_threshold integer,
    value_threshold numeric(16,2),
    primary key(id)
    );
');

perform ex(E'
    create table loyalty_control(
    id serial,
    max_reward_div numeric(16,2),
    product_type_id int,
    cash_points_ratio int,
    points_cash_ratio numeric(16,2),
    primary key(id)
    );
');

perform ex(E' ALTER TABLE sigcap_queue ADD COLUMN customer_exception boolean DEFAULT false; ');
perform ex(E' ALTER TABLE sigcap_queue ADD COLUMN driver_exception boolean DEFAULT false; ');

perform ex(E'
    CREATE TABLE mobile_return_driver_text
    (
    id serial NOT NULL,
    mobile_return_id integer NOT NULL,
    note text NOT NULL,
    CONSTRAINT mobile_return_driver_text_pk PRIMARY KEY (id),
    CONSTRAINT mobile_return_driver_text_mr_fk FOREIGN KEY (mobile_return_id)
    REFERENCES mobile_return (id) MATCH SIMPLE
    ON UPDATE NO ACTION ON DELETE NO ACTION
    )
    WITH (
    OIDS=FALSE
);
');

perform ex(E' ALTER TABLE mobile_return_driver_text OWNER TO postgres; ');


perform ex(E'
    CREATE TABLE depot_ext
    (
      id serial NOT NULL,
      depot_id integer,
      stock_location integer,
      CONSTRAINT depot_ext_pk PRIMARY KEY (id ),
      CONSTRAINT depot_ext_to_depot FOREIGN KEY (depot_id)
          REFERENCES depot (cod) MATCH SIMPLE
          ON UPDATE NO ACTION ON DELETE NO ACTION
    )
    WITH (
      OIDS=FALSE
    );

');

perform ex(E' ALTER TABLE depot_ext OWNER TO postgres; ');

end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misctables() OWNER TO postgres;

select * from misctables();


